from __future__ import unicode_literals

from django import forms
from django.contrib.auth.forms import (ReadOnlyPasswordHashField,
                                       AuthenticationForm,
                                       PasswordResetForm, 
                                       SetPasswordForm, 
                                       PasswordChangeForm, 
                                       AdminPasswordChangeForm)
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model


class UserCreationForm(forms.ModelForm):
    """
    A form that creates a user, with no privileges, from the given email and
    password.
    It could be used also for derived MailUser models, it gets User model from
    settings.AUTH_USER_MODEL .
    """
    error_messages = {
        'duplicate_email': _("A user with that email already exists."),
        'password_mismatch': _("The two password fields didn't match."),
    }
    email = forms.EmailField(label=_("Email"), max_length=254,
        help_text=_("Required. Valid email for registration and login."),
        error_messages={
            'invalid': _("This value must be a valid email address.")})
    password1 = forms.CharField(label=_("Password"),
        widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password confirmation"),
        widget=forms.PasswordInput,
        help_text=_("Enter the same password as above, for verification."))

    class Meta:
        model = get_user_model()
        fields = ("email",)

    def clean_email(self):
        # Since User.email is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        email = self.cleaned_data["email"]
        try:
            get_user_model()._default_manager.get(email=email)
        except get_user_model().DoesNotExist:
            return email
        raise forms.ValidationError(self.error_messages['duplicate_email'])

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'])
        return password2

    def save(self, commit=True):
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user

class UserChangeForm(forms.ModelForm):
    """
    Changes user email and password
    """
    email = forms.EmailField(
        label=_("Email"), max_length=254,
        help_text=_("Required. 30 characters or fewer. Letters, digits and "
                      "@/./+/-/_ only."),
        error_messages={
            'invalid': _("This value may contain only letters, numbers and "
                         "@/./+/-/_ characters.")})
    password = ReadOnlyPasswordHashField(label=_("Password"),
        help_text=_("Raw passwords are not stored, so there is no way to see "
                    "this user's password, but you can change the password "
                    "using <a href=\"password/\">this form</a>."))

    class Meta:
        model = get_user_model()
        fields = ('email',)

    def __init__(self, *args, **kwargs):
        super(UserChangeForm, self).__init__(*args, **kwargs)
        f = self.fields.get('user_permissions', None)
        if f is not None:
            f.queryset = f.queryset.select_related('profile_type')

    def clean_password(self):
        # Regardless of what the user provides, return the initial value.
        # This is done here, rather than on the field, because the
        # field does not have access to the initial value
        return self.initial["password"]


class AuthenticationForm(AuthenticationForm):
    """
    Base class for authenticating users. Extend this to get a form that accepts
    username/password logins.
    
    Derives auth.contrib.AuthenticationForm and it is usable for
    username password sematic
    """
    pass


class PasswordResetForm(PasswordResetForm):
    """
    Derives auth.contrib.PasswordResetForm and it is usable for
    mail password sematic
    """
    pass


class SetPasswordForm(SetPasswordForm):
    """
    A form that lets a user change set his/her password without entering the
    old password
    """
    pass

class PasswordChangeForm(PasswordChangeForm):
    """
    A form that lets a user change his/her password by entering
    their old password.
    """
    pass


class AdminPasswordChangeForm(AdminPasswordChangeForm):
    """
    A form used to change the password of a user in the admin interface.
    """
    pass

class AdminUserPasswordChangeForm(UserChangeForm):
    """ 
    A form to change the username and the password simultaneously
    without asking the previous password to the user
    """
    password = forms.CharField(label=_("Password"),
                               help_text=_("Leave empty to not update the password"),
                               required=False,
                               widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password (again)"),
                                required=False,
                                widget=forms.PasswordInput)

    def clean_password(self):
        password = self.cleaned_data.get('password')
        password2 = self.cleaned_data.get('password2')
        if password and password2:
            if password != password2:
                raise forms.ValidationError(
                    self.error_messages['password_mismatch'],
                    code='password_mismatch',
                )
        return password

    def save(self, commit=True):
        """
        Saves the new password.
        """
        if self.cleaned_data['password']:
            self.instance.set_password(self.cleaned_data["password"])
            if commit:
                self.instance.save()
        return super(AdminUserPasswordChangeForm, self).save(commit)


try:
    from .models import FollowsConfirmed

    class FollowConfirmUserForm(forms.Form):
        """
        A form that permits another user to follow another
        """
        email = forms.EmailField(help_text=_('The user email address'))

        def __init__(self, check_existence=True, *args, **kwargs):
            super(FollowConfirmUserForm, self).__init__(*args, **kwargs)
            self._check_existence = check_existence

        def clean_email(self):
            email = self.cleaned_data['email']
            if self._check_existence:
                try:
                    user = get_user_model().objects.get(email=self.cleaned_data['email'])
                except get_user_model().DoesNotExist:
                    self._errors['email'] = _('No user with this email exists')
            return email

        def save(self, commit=True, follower=None, mutual=False, 
                 askconfirm=True, followeraskconfirm=False):
            if not follower:
                raise ValueError("follower parameter must not be None")
            user = get_user_model().objects.get(email=self.cleaned_data.get('email', None))
            instance_r = FollowsConfirmed(follower=follower, followed=user)
            if commit:
                instance_r.save(askconfirm)
                if mutual:
                    instance_d = FollowsConfirmed(follower=user, followed=follower)
                    instance_d.save(followeraskconfirm)
            return instance_r


    class FollowConfirmUserWithpasswordForm(FollowConfirmUserForm):
        """
        A form that permits another user to follow another one providing his password
        """
        password = forms.CharField(widget=forms.PasswordInput)

        def __init__(self, check_password=True, *args, **kwargs):
            super(FollowConfirmUserWithpasswordForm, self).__init__(*args, **kwargs)
            self._check_password = check_password
            if not check_password:
                self.fields['password'].required = False
                self.fields['password'].help_text = _('The password will not be checked if the user exists, insert only the user email')

        def clean(self):
            cleaned_data = super(FollowConfirmUserWithpasswordForm, self).clean()
            try:
                user = get_user_model().objects.get(email=cleaned_data.get('email', None))
                if not self._check_password or (user and user.check_password(cleaned_data.get('password', None))):
                    return cleaned_data 
                else:
                    cleaned_data['password'] = ""
                    self._errors['password'] = [_('Your username and password didn\'t match.')]
            except get_user_model().DoesNotExist:
                pass

            return cleaned_data 

except ImportError:
    pass



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model

from .models import MailUser, MailSocialUser
from .forms import UserCreationForm, UserChangeForm

class MailUserAdmin(UserAdmin):
    form = UserChangeForm
    add_form = UserCreationForm

    # The forms to add and change user instances
    fieldsets = (
        (None, {'fields': ('email', 'password',)}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser',
                                       'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('email', 'password1', 'password2')}
        ),
    )
    list_display = ('email', 'is_staff')
    search_fields = ('email', 'is_staff')
    ordering = ('email',)


class MailSocialUserAdmin(MailUserAdmin):
    fieldsets = MailUserAdmin.fieldsets + (
        (_('Social interactions'), {'fields': ('follows', 'likes')}),
    )


admin.site.register(MailUser, MailUserAdmin)
admin.site.register(MailSocialUser, MailSocialUserAdmin)


try: 
    from .models  import MailConfirmedUser, MailConfirmedSocialUser, MailSocialConfirmfollowUser, MailConfirmedSocialConfirmfollowUser


    class MailConfirmedUserAdmin(MailUserAdmin):
        fieldsets = MailUserAdmin.fieldsets + (
            (_('Confirmed'), {'fields': ('confirmed',)}),
        )
        list_display = MailUserAdmin.list_display + ('confirmed',)


    class MailConfirmedSocialUserAdmin(MailConfirmedUserAdmin):
        fieldsets = MailConfirmedUserAdmin.fieldsets + (
            (_('Social interactions'), {'fields': ('follows', 'likes')}),
        )


    class FollowsInline(admin.TabularInline):
        model = FollowsConfirmed
        fields = ['date', 'follower', 'followed', 'confirmed']


    class MailSocialConfirmfollowUserAdmin(MailSocialUserAdmin):
        inlines = ['FollowsInline',]


    class MailConfirmedSocialConfirmfollowUserAdmin(MailConfirmedUserAdmin):
        inlines = ['FollowsInline',]


    admin.site.register(MailConfirmedUser, MailConfirmedUserAdmin)
    admin.site.register(MailConfirmedSocialUser, MailConfirmedSocialUserAdmin)

    admin.site.register(MailSocialConfirmfollowUser, MailSocialConfirmfollowUserAdmin)
    admin.site.register(MailConfirmedSocialConfirmfollowUser, MailConfirmedSocialConfirmfollowUserAdmin)


except:
    pass



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.mail import send_mail
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import (AbstractBaseUser,
                                        PermissionsMixin,
                                        BaseUserManager)
from django.dispatch import receiver
from django.db import models
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.http import urlquote
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


def _getSettingKey(key, default):
    """
    Get a key's value defined in settings
    """
    getattr(settings, key, default)


class MailUserManager(BaseUserManager):

    def create_user(self, email, password=None, **extra_fields):
        """
        Creates and saves a User with the given email and password.

        """
        now = timezone.now()
        if not email:
            raise ValueError('The given email must be set')
        email = MailUserManager.normalize_email(email)
        user = self.model(email=email,
                          is_staff=False, is_active=True, is_superuser=False,
                          last_login=now, date_joined=now, **extra_fields)

        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password, **extra_fields):
        u = self.create_user(email, password, **extra_fields)
        u.is_staff = True
        u.is_active = True
        u.is_superuser = True
        u.save(using=self._db)
        return u


@python_2_unicode_compatible
class AbstractMailUser(AbstractBaseUser, PermissionsMixin):
    """
    An abstract base class implementing fully featured User model with
    admin-compliant permission

    Email, password and email are required. Email is identifying the User
    and it is unique.
    """

    email = models.EmailField(_('email'), max_length=254, unique=True,
                              help_text=_('Required. Email field'),
                              error_messages={'unique': _('This email is not available, choose another one')})
    is_staff = models.BooleanField(_('staff status'), default=False,
        help_text=_('Designates whether the user can log into this admin '
                    'site.'))
    is_active = models.BooleanField(_('active'), default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    last_modified = models.DateTimeField(_('last modified'), auto_now=True)

    # This three fields permits to have a related
    # profile associated with this user
    profile_type = models.ForeignKey(ContentType, null=True, blank=True)
    profile_id = models.PositiveIntegerField(blank=True, null=True)
    profile = generic.GenericForeignKey('profile_type', 'profile_id')

    objects = MailUserManager()

    USERNAME_FIELD = 'email'

    def __str__(self):
        return self.get_full_name()

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def get_absolute_url(self, language=None):
        return "/users/%s/" % urlquote(self.id)

    def get_full_name(self):
        try:
            return force_text(self.get_profile())
        except:
            return self.email

    def get_short_name(self):
        "Returns the short name for the user."
        return self.email.split("@")[0]

    def email_user(self, subject, message, from_email=None):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email])

    def get_profile(self):
        if hasattr(self, '_cached_profile'):
            return self._cached_profile
        try:
            # altuser profiles is a list of reverse relation names
            # of what could be your different site profiles
            for p in settings.ALTUSER_PROFILES_BREL:
                try:
                    self._cached_profile = getattr(self, p)
                    return self._cached_profile
                except ObjectDoesNotExist:
                    pass
        except AttributeError:
            # if ALTUSER_PROFILES_BREL does not exists
            if self.profile is not None:
                self._cached_profile = self.profile
                return self._cached_profile
        raise NotImplementedError('Not implemented')

    @staticmethod
    def generate_username():
        import uuid
        try:
            from django.contrib.sites.models import Site
            site = Site.objects.get_current().domain
        except:
            site = None
        site = site if site else "example.com"
        return uuid.uuid4().hex + "@" + site


class MailUser(AbstractMailUser):
    """
    User model with email as required identification field and without username.

    Email and password are required.
    """
    class Meta:
        swappable = 'AUTH_USER_MODEL'


class MailSocialUser(AbstractMailUser):
    """
    User model with email as required identification field and without username.

    Email and password are required.
    It also provides follows and likes fields to track this kind of user interaction.
    """
    likes = models.ManyToManyField('self', blank=True, related_name='liked', symmetrical=False)
    follows = models.ManyToManyField('self', blank=True, related_name='followed', symmetrical=False)

    class Meta:
        swappable = 'AUTH_USER_MODEL'


try:
    from mail_confirmation.models import MailConfirmation
    from mail_confirmation.utils import generate_and_send, generate_confirmation

    class AbstractMailConfirmedUser(models.Model):
        confirmed = generic.GenericRelation(MailConfirmation,
                                            content_type_field='toconfirm_type',
                                            object_id_field='toconfirm_id')

        def create_user(self, email, password=None, askconfirm=True, **extra_fields):
            user = super(AbstractMailConfirmedUser, self).create_user(email, password, **extra_fields)
            user.is_active = not askconfirm
            if askconfirm:
                generate_and_send(user, 
                                  request_template='altuser/mail_account_request.html',
                                  success_template='altuser/mail_account_confirmation_success.html',
                                  subject=_('User registration confirmation'),
                                  to=user.email,
                                  )
            else:
                generate_confirmation(self, confirmed=True)
            return user

        class Meta:
            abstract = True


    from mail_confirmation.signals import confirmed_signal

    @receiver(confirmed_signal, sender=MailConfirmation)
    def activateuser_on_confirmation(sender, toconfirm_type, object_id, **kwargs):
        for base in toconfirm_type.__bases__:
            if base == AbstractMailConfirmedUser: 
                toconfirm_type.objects.get(id=object_id).is_active = True
                break

    class AbstractConfirmfollowUser(models.Model):
        likes = models.ManyToManyField('self', related_name='liked', symmetrical=False)
        follows = models.ManyToManyField('self', through='FollowsConfirmed', symmetrical=False)

        class Meta:
            swappable = 'AUTH_USER_MODEL'
            abstract = True

    class FollowsConfirmed(models.Model):
        """
        Used as a though relation form followconfirmed models
        ask confirmation to the followed user sending an email
        """
        date = models.DateTimeField(verbose_name=_('followed on'),
                                    auto_now=True)
        follower = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='relations')
        followed = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='+')
        confirmed = generic.GenericRelation(MailConfirmation,
                                            content_type_field='toconfirm_type',
                                            object_id_field='toconfirm_id')

        class Meta:
            unique_together = ("followed", "follower")

        def save(self, askconfirm=True, *args, **kwargs):
            super(FollowsConfirmed, self).save(*args, **kwargs)
            if not self.id:
                self.gen_confirmation(askconfirm)

        def gen_confirmation(self, askconfirm):
            if askconfirm:
                generate_and_send(self,
                                  request_template='altuser/mail_follow_request.html',
                                  success_template='altuser/mail_follows_confirmation_success.html',
                                  subject=_('Follow request'),
                                  to=self.followed.email,
                )
            else:
                generate_confirmation(self, confirmed=True)

    class MailSocialConfirmfollowUser(AbstractMailUser, AbstractConfirmfollowUser):
        """
        User model with email as required identification field and without username.

        Email and password are required.
        It also provides follows and likes fields to track this kind of user interaction.

        It has a confirmation facility for the follow action, the user followed must confirm
        the request.
        """
        class Meta:
            swappable = 'AUTH_USER_MODEL'


    class MailConfirmedSocialConfirmfollowUser(AbstractMailConfirmedUser, AbstractConfirmfollowUser, AbstractMailUser):
        """
        User model with email as required identification field and without username.

        Email and password are required.
        It also provides follows and likes fields to track this kind of user interaction.

        Uses email as confirmation to the user subscription

        It has a confirmation facility for the follow action, the user followed must confirm 
        the request.
        """
        class Meta:
            swappable = 'AUTH_USER_MODEL'


    class MailConfirmedUser(AbstractMailConfirmedUser, AbstractMailUser):
        """
        User model with email as required identification field and without username.

        Email and password are required.

        Uses email as confirmation to the user subscription
        """

        class Meta:
            swappable = 'AUTH_USER_MODEL'


    class MailConfirmedSocialUser(AbstractMailConfirmedUser, AbstractMailUser):
        """
        User model with email as required identification field and without username.

        Email and password are required.
        It also provides follows and likes fields to track this kind of user interaction.

        Uses email as confirmation to the user subscription
        """
        likes = models.ManyToManyField('self', related_name='liked', symmetrical=False)
        follows = models.ManyToManyField('self', related_name='followed', symmetrical=False)

        class Meta:
            swappable = 'AUTH_USER_MODEL'

except:
        pass
